<?php

declare(strict_types=1);

namespace BitBag\SyliusBonusPointsPlugin\Checker\Rule;

use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategyInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\OrderItemInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Webmozart\Assert\Assert;

final class AppliedOnceRuleChecker implements BonusPointsStrategyRuleCheckerInterface
{
    private RepositoryInterface $bonusPointsStrategyRepository;

    private RepositoryInterface $customerBonusPointsStrategyRepository;

    public function __construct(
        RepositoryInterface $bonusPointsStrategyRepository,
        RepositoryInterface $customerBonusPointsStrategyRepository
    ) {
        $this->bonusPointsStrategyRepository = $bonusPointsStrategyRepository;
        $this->customerBonusPointsStrategyRepository = $customerBonusPointsStrategyRepository;
    }

    public function isEligible(OrderItemInterface $orderItem, array $configuration): bool
    {
        Assert::keyExists($configuration, 'strategyCode');

        /** @var OrderInterface $order */
        $order = $orderItem->getOrder();
        Assert::isInstanceOf($order, OrderInterface::class);

        /** @var CustomerInterface $customer */
        $customer = $order->getCustomer();
        if (!$customer instanceof CustomerInterface) {
            return true;
        }

        /** @var BonusPointsStrategyInterface $bonusPointsStrategy */
        $bonusPointsStrategy = $this->bonusPointsStrategyRepository->findOneBy(['code' => $configuration['strategyCode']]);
        Assert::isInstanceOf($bonusPointsStrategy, BonusPointsStrategyInterface::class);

        return null === $this->customerBonusPointsStrategyRepository->findOneBy([
            'customer' => $customer,
            'bonusPointsStrategy' => $bonusPointsStrategy,
        ]);
    }
}
