<?php

declare(strict_types=1);

namespace BitBag\SyliusBonusPointsPlugin\Form\Type;

use BitBag\SyliusBonusPointsPlugin\Calculator\BonusPointsStrategyCalculatorInterface;
use Sylius\Component\Registry\ServiceRegistryInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\ChoiceList\View\ChoiceView;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolver;

final class BonusPointsStrategyCalculatorChoiceType extends AbstractType
{
    /** @var ServiceRegistryInterface */
    private $registry;

    /** @var array */
    private $calculators;

    public function __construct(
        ServiceRegistryInterface $registry,
        array $calculators
    ) {
        $this->registry = $registry;
        $this->calculators = $calculators;
    }

    public function buildView(FormView $view, FormInterface $form, array $options)
    {
        $view->vars['choices'] = $this->resolveCalculators($options['strategy_type']);
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefault('choices', array_flip($this->calculators));
        $resolver->setRequired('strategy_type');
        $resolver->setAllowedTypes('strategy_type', 'string');
    }

    public function getParent(): string
    {
        return ChoiceType::class;
    }

    private function resolveCalculators(string $type): array
    {
        $choices = array_filter(array_flip($this->calculators), function ($calculatorServiceName) use ($type) {
            /** @var BonusPointsStrategyCalculatorInterface $calculator */
            $calculator = $this->registry->get($calculatorServiceName);

            return in_array($type, $calculator->getCompatibleStrategyTypes());
        });

        array_walk($choices, function (&$choice, $label) {
            $choice = new ChoiceView($choice, $choice, $label);
        });

        return $choices;
    }
}
